@@ -5,7 +5,7 @@ use warnings;
 
 package Class::Tiny;
 # ABSTRACT: Minimalist class construction
-our $VERSION = '0.007'; # VERSION
+our $VERSION = '0.008'; # VERSION
 
 use Carp ();
 
@@ -77,15 +77,26 @@ sub get_all_attribute_defaults_for {
 
 package Class::Tiny::Object;
 # ABSTRACT: Base class for classes built with Class::Tiny
-our $VERSION = '0.007'; # VERSION
+our $VERSION = '0.008'; # VERSION
 
-my %CAN_CACHE;
-my %LINEAR_ISA_CACHE;
-my %DEMOLISH_CACHE;
-my %BUILD_CACHE;
+my ( %LINEAR_ISA_CACHE, %BUILD_CACHE, %DEMOLISH_CACHE, %CAN_CACHE );
+
+my $_PRECACHE = sub {
+    my ($class) = @_;
+    $LINEAR_ISA_CACHE{$class} =
+      @{"$class\::ISA"} == 1 && ${"$class\::ISA"}[0] eq "Class::Tiny::Object"
+      ? [$class]
+      : mro::get_linear_isa($class);
+    for my $s ( @{ $LINEAR_ISA_CACHE{$class} } ) {
+        $BUILD_CACHE{$s}    = *{"$s\::BUILD"}{CODE};
+        $DEMOLISH_CACHE{$s} = *{"$s\::DEMOLISH"}{CODE};
+    }
+    return $LINEAR_ISA_CACHE{$class};
+};
 
 sub new {
     my $class = shift;
+    my $linear_isa = $LINEAR_ISA_CACHE{$class} || $_PRECACHE->($class);
 
     # handle hash ref or key/value arguments
     my $args;
@@ -103,14 +114,9 @@ sub new {
 
     # create object and invoke BUILD
     my $self = bless {%$args}, $class;
-    my $linear_isa = $LINEAR_ISA_CACHE{$class} ||=
-      @{"$class\::ISA"} == 1 && ${"$class\::ISA"}[0] eq "Class::Tiny::Object"
-      ? [$class]
-      : mro::get_linear_isa($class);
-
     for my $s ( reverse @$linear_isa ) {
-        my $builder = $BUILD_CACHE{$s} ||= *{"$s\::BUILD"}{CODE};
-        $builder->( $self, $args ) if $builder;
+        next unless my $builder = $BUILD_CACHE{$s};
+        $builder->( $self, $args );
     }
 
     # unknown attributes still in $args are fatal
@@ -135,8 +141,7 @@ sub DESTROY {
       ? ${^GLOBAL_PHASE} eq 'DESTRUCT'
       : Devel::GlobalDestruction::in_global_destruction();
     for my $s ( @{ $LINEAR_ISA_CACHE{$class} } ) {
-        my $demolisher = $DEMOLISH_CACHE{$s} ||= *{"$s\::DEMOLISH"}{CODE};
-        next unless $demolisher;
+        next unless my $demolisher = $DEMOLISH_CACHE{$s};
         my $e = do {
             local ( $?, $@ );
             eval { $demolisher->( $self, $in_global_destruction ) };
@@ -164,7 +169,7 @@ Class::Tiny - Minimalist class construction
 
 =head1 VERSION
 
-version 0.007
+version 0.008
 
 =head1 SYNOPSIS
 
@@ -199,7 +204,7 @@ In F<example.pl>:
 
 =head1 DESCRIPTION
 
-This module offers a minimalist class construction kit in around 100 lines of
+This module offers a minimalist class construction kit in around 120 lines of
 code.  Here is a list of features:
 
 =over 4
