@@ -5,6 +5,6 @@ use warnings;
 package Foxtrot;
 
 use Class::Tiny 'foo';
-use Class::Tiny 'bar';
+use Class::Tiny { bar => 42, baz => sub { time } };
 
 1;
